/*
 * RRT.cpp
 *
 *  Created on: Nov 20, 2012
 *      Author: ddevaurs
 */

#include "Planner-pkg.h"

#include "API/Roadmap/compco.hpp"
#include "API/Roadmap/graph.hpp"

#include "planner/planEnvironment.hpp"

#include "planner/Diffusion/RRT.hpp"

using namespace std;


/**
 * ****************************************************
 *      RRT Expansion
 * ****************************************************
 */


/**
 * Sample a direction for the expansion process (taking into account the goal bias, if relevant).
 * @param directionNode: contains the goal node when relevant
 * @return the configuration toward which the tree will be expanded
 */
confPtr_t RRTExpansion::sampleExpansionDirection(Node * directionNode)
{
    // bias toward the goal configuration
    if (ENV.getBool(Env::isGoalBiased) && p3d_random(0., 1.) <= ENV.getDouble(Env::Bias))
    {
        directionNode = goalNode;
        return goalNode->getConfiguration();
    }
    // or random sampling in the entire C-Space
    else
        return m_Graph->getRobot()->shoot();
}

confPtr_t RRTExpansion::sampleExpansionDirection(Node * directionNode, Node *fromNode)
{
    // bias toward the goal configuration
    if (ENV.getBool(Env::isGoalBiased) && p3d_random(0., 1.) <= ENV.getDouble(Env::Bias))
    {
        directionNode = goalNode;
        return goalNode->getConfiguration();
    }
    // or random sampling in the entire C-Space
    else
    {
        return m_Graph->getRobot()->shootDirections(fromNode->getConfiguration(),(int)p3d_random(1.,8.));
    }
}


/**
 * RRT Expand process: call the appropriate expansion method.
 * @param fromNode: node from which the expansion is performed
 * @param directionConf: configuration toward which the expansion is going
 * @param directionNode: node toward which the expansion is going (when relevant)
 * @param method: Extend or Connect
 * @return the number of created nodes
 */
unsigned RRTExpansion::expandProcess(Node * fromNode, confPtr_t directionConf, Node * directionNode,
        Env::expansionMethod method)
{
    switch (method)
    {
    case Env::Extend:
        return this->extend(fromNode, directionConf, directionNode);

    case Env::Connect:
        return this->connect(fromNode, directionConf, directionNode);

    default:
        cerr << "*** ERROR: RRT-Expansion::expandProcess --> method not implemented !!! ***" << endl;
        return 0;
    }
}


/**
 * Attempt to link two given nodes (i.e. two components, when relevant).
 * @return TRUE if the junction is successful.
 */
bool RRTExpansion::attemptJunction(Node * fromNode, Node * toNode)
{
    // path along which the junction is attempted
    LocalPath path(fromNode->getConfiguration(), toNode->getConfiguration());

    // N.B.: the connection is attempted only if the path length is less than a specified threshold
    // (which is some multiple of the extension step)
    if (path.getParamMax() <= ENV.getDouble(Env::minimalFinalExpansionGap) * this->step())
    {
        // check whether the path is valid (i.e. collision-free)
        return path.isValid();
    }

    return false;
}


/**
 * RRT Extend.
 * @param fromNode: node from which the expansion is performed
 * @param directionConf: configuration toward which the expansion is going
 * @param directionNode: node toward which the expansion is going (when relevant)
 * @return the number of created nodes
 */
unsigned RRTExpansion::extend(Node * fromNode, confPtr_t directionConf, Node * directionNode)
{

	confPtr_t fromConf = fromNode->getConfiguration();

    // path along which the extension is performed
    LocalPath directionPath(fromConf, directionConf);
    double k = this->computeInterpolationFactor(directionPath.getParamMax());

    // construct an 'extension path', by linear interpolation along the 'direction path'
    LocalPath extensionPath(fromConf, this->interpolate(directionConf, &directionPath, k));
    if(directionPath.getLocalpathStruct()->type_lp==QUADSPLINE){
        p3d_localpath *localpathPt=extensionPath.getLocalpathStruct();
        p3d_rob *robotPt=fromNode->getRobot()->getRobotStruct();
        if(localpathPt && robotPt){
            lm_quadspline_str *quadspline_paramPt=lm_get_quadspline_lm_param(robotPt);
            p3d_quadspline_data *quadspline_data=localpathPt->specific.quadspline_data;
            if(quadspline_paramPt && quadspline_data){
                configPt q_f=quadspline_data->q_end;
                for(int i=0;i<robotPt->nb_dof;i++){
                    q_f[i]=directionPath.getLocalpathStruct()->specific.quadspline_data->q_end[i];
                }
                set_qf_voro_opt(robotPt,localpathPt,k,0.01);
            }
        }
    }

    // if the path is valid, add a new node to the graph
    if (extensionPath.isValid())
    {
        unsigned nbCreatedNodes(0);
        this->addNode(fromNode, extensionPath, k, directionNode, nbCreatedNodes);
    	return nbCreatedNodes;
    }
    else {
		this->expansionFailed(*fromNode);
		return 0;
    }
}


/**
 * RRT Connect.
 * @param fromNode: node from which the expansion is performed
 * @param directionConf: configuration toward which the expansion is going
 * @param directionNode: node toward which the expansion is going (when relevant)
 * @return the number of created nodes
 */
unsigned RRTExpansion::connect(Node * fromNode, confPtr_t directionConf, Node * directionNode)
{
    confPtr_t fromConf = fromNode->getConfiguration();

    // path along which the connection is performed
    LocalPath directionPath(fromConf, directionConf);
    double k = this->computeInterpolationFactor(directionPath.getParamMax());

    bool success(true);
    confPtr_t currentConf = fromConf;
    confPtr_t newConf;
    double prop(k);
    double oldProp(0.);

    // attempt the connection step by step
    while (success && prop <= 1.)
    {
        // check whether the current extension path is valid (i.e. without collision)
    	newConf = this->interpolate(directionConf, &directionPath, prop);
        LocalPath extensionPath(currentConf, newConf);
        if (!extensionPath.isValid())
            success = false;
        else
        {
            currentConf = newConf;
            oldProp = prop;
            if (prop < 1.)
                prop = min(1., prop + k);
            else
                prop = 2.;
        }
    }

    // if "reached" or "advanced", add a node to the graph
    if (success || prop > k)
    {
        LocalPath connectPath(fromConf, currentConf);
        unsigned nbCreatedNodes(0);
        this->addNode(fromNode, connectPath, oldProp, directionNode, nbCreatedNodes);
        return nbCreatedNodes;
    }
    else {
    	this->expansionFailed(*fromNode);
    	return 0;
    }
}


/**
 * ****************************************************
 *             RRT
 * ****************************************************
 */

/**
 * Initialize the RRT.
 * @return the number of nodes added to the graph
 */
unsigned RRT::init()
{
	// create the RRT expansion
    expansion = new RRTExpansion(_Graph, _Goal);

    // initialize the Tree Planner
    return TreePlanner::init();
}


/**
 * Main function of the RRT.
 * @return the number of nodes added to the graph
 */
unsigned RRT::run()
{
    double ts(0.);
    m_time = 0.;
    ChronoOn();

	// check the pre-conditions
	if (!this->preConditions()) {
		cout << "Failure: pre-conditions are not satisfied." << endl;
		return 0;
	}

	// trivial case
	if (ENV.getBool(Env::expandToGoal) && this->connectNodeToComp(_Start, _Goal)) {
		cout << "Trivial: Direct connection between Start and Goal." << endl;
		return 0;
	}

	unsigned nbCreatedNodes(0); // number of nodes created by one expansion step
	unsigned totalCreatedNodes(0); // total number of nodes created so far
	Node * fromNode = _Start; // node belonging to the component that will be expanded
    Node * toNode = _Goal; // node belonging to the component toward which to expand (if relevant)

	// RRT loop: start by checking the stopping conditions
	while (!this->checkStopConditions())
	{
		// in the case of a balanced Bi-directional RRT, do not expand when components are unbalanced
		if (!ENV.getBool(Env::biDir) || !ENV.getBool(Env::expandBalanced)
				|| (fromNode->getConnectedComponent()->getNumberOfNodes()
						< toNode->getConnectedComponent()->getNumberOfNodes() + 5))
		{
			++m_nbExpansion;

            // perform a single expansion step of RRT
            nbCreatedNodes = this->expandOneStep(fromNode);

			// success, if some nodes have been created
			if (nbCreatedNodes > 0)
            {
				totalCreatedNodes += nbCreatedNodes;
				m_nbConscutiveFailures = 0;

				// attempt a connection toward the goal / the other component (when relevant)
				if (ENV.getBool(Env::expandToGoal))
				{
					// Bi-directional RRT
					if (ENV.getBool(Env::biDir))
					{
						// look for the node whose configuration is the closest to that of the last node
						// within the connected component to which toNode belongs
						Node * closestNode = toNode->getConnectedComponent()->nearestNeighbour(
								_Graph->getLastNode()->getConfiguration());
						this->connectNodeToComp(_Graph->getLastNode(), closestNode);
					}
					// mono-directional RRT (with goal)
					else
						this->connectNodeToComp(_Graph->getLastNode(), _Goal);
				}

				// redraw the scene (when relevant)
				if (!ENV.getBool(Env::drawDisabled) && ENV.getBool(Env::drawExploration))
					_draw_func();
			}
			// failure, otherwise
			else {
				m_nbFailedExpansion++;
                m_nbConscutiveFailures++;
			}
		}
		ChronoTimes(&m_time, &ts);

		// in the case of the Bi-directional RRT, swap the roles of the connected components
		if (ENV.getBool(Env::biDir))
			swap(fromNode, toNode);
	}

	ChronoOff();
	return totalCreatedNodes;
}


/**
 * Check whether RRT has found a trajectory.
 */
bool RRT::trajFound()
{
	// success in case of a search involving a goal: the Start and Goal are in the same component
	return (ENV.getBool(Env::expandToGoal) && _Start->inSameComponent(_Goal));
}


/**
 * Check the stopping conditions of RRT.
 * @return TRUE if one of the conditions is satisfied, and FALSE otherwise
 */
bool RRT::checkStopConditions()
{
    if (PlanEnv->getBool(PlanParam::stopPlanner)) {
        cout << "Stop: planning canceled by user." << endl;
        return true;
    }

    if (PlanEnv->getBool(PlanParam::planWithTimeLimit) &&
            m_time > PlanEnv->getDouble(PlanParam::timeLimitPlanning)) {
        cout << "Failure: the time limit has been reached (" << m_time << " s)." << endl;
        return true;
    }

    if (m_nbExpansion > (unsigned) PlanEnv->getInt(PlanParam::plannerMaxIterations)) {
        cout << "Failure: the maximum number of expansions has been reached." << endl;
        return true;
    }

    if (m_nbConscutiveFailures > (unsigned) ENV.getInt(Env::NbTry)) {
        cout << "Failure: the maximum number of consecutive failures has been reached." << endl;
        return true;
    }

    if (_Graph->getNumberOfNodes() > (unsigned) ENV.getInt(Env::maxNodeCompco)) {
        cout << "Failure: the maximum number of nodes has been reached." << endl;
        return true;
    }

    return this->trajFound();
}


/**
 * Perform a single expansion step of RRT, growing the connected component containing fromNode.
 * @return the number of created nodes
 */
unsigned RRT::expandOneStep(Node * fromNode)
{
	// sample the expansion direction (i.e. the configuration toward which the expansion will be performed)
	// N.B.: if the expansion is performed toward a node than already exists (e.g. the goal node),
	//       this node becomes the dirNode and dirConf simply points to its configuration
	Node * dirNode(NULL);
	confPtr_t dirConf = expansion->sampleExpansionDirection(dirNode);

	// select, from the component to be grown, the node from which the expansion will be attempted
	Node * expandNode = fromNode->getConnectedComponent()->nearestNeighbour(dirConf);

	// directed expansion
    return expansion->expandProcess(expandNode, dirConf, dirNode, ENV.getExpansionMethod());
}


/**
 * Try to link a given node to a given component (accessed through the node compNode).
 * @return TRUE if the connection is successful.
 */
bool RRT::connectNodeToComp(Node * node, Node * compNode)
{
	// if the junction is successful, update the graph
	if (expansion->attemptJunction(node, compNode))
	{
		if (node->getConnectedComponent() != compNode->getConnectedComponent())
            return _Graph->linkNodeAndMerge(node, compNode,true);
		else {
			_Graph->addEdges(node, compNode);
			return true;
		}
	}
	else
		return false;
}
