/*
 * RRT.hpp
 *
 *  Created on: Nov 20, 2012
 *      Author: ddevaurs
 */

#ifndef _RRT_HPP_
#define _RRT_HPP_

#include "planner/Diffusion/TreePlanner.hpp"
#include "planner/Diffusion/Variants/BaseExpansion.hpp"


/**
 * Expansion procedure of the Rapidly-exploring Random Tree (RRT) algorithm.
 */
class RRTExpansion: public BaseExpansion
{
public:
	//! Constructor.
	RRTExpansion(Graph * graph, Node * goal) : BaseExpansion(graph), goalNode(goal) {}	;

	//! Destructor.
	virtual ~RRTExpansion() {	};

	/**
	 * Sample a direction for the expansion process (taking into account the goal bias, if relevant).
	 * @param directionNode: contains the goal node when relevant
	 * @return the configuration toward which the tree will be expanded
	 */
    virtual confPtr_t sampleExpansionDirection(Node * directionNode);

    virtual confPtr_t sampleExpansionDirection(Node * directionNode,Node* fromNode);

	/**
	 * RRT Expand process: call the appropriate expansion method.
	 * @param fromNode: node from which the expansion is performed
	 * @param directionConf: configuration toward which the expansion is going
	 * @param directionNode: node toward which the expansion is going (when relevant)
	 * @param method: Extend or Connect
	 * @return the number of created nodes
	 */
	unsigned expandProcess(Node * fromNode, confPtr_t directionConf, Node * directionNode,
			Env::expansionMethod method);

	/**
	 * Attempt to link two given nodes (i.e. two components, when relevant).
	 * @return TRUE if the junction is successful.
	 */
	virtual bool attemptJunction(Node * fromNode, Node * toNode);

private:
	/**
	 * RRT Extend.
	 * @param fromNode: node from which the expansion is performed
	 * @param directionConf: configuration toward which the expansion is going
	 * @param directionNode: node toward which the expansion is going (when relevant)
	 * @return the number of created nodes
	 */
	virtual unsigned extend(Node * fromNode, confPtr_t directionConf, Node * directionNode);

	/**
	 * RRT Connect.
	 * @param fromNode: node from which the expansion is performed
	 * @param directionConf: configuration toward which the expansion is going
	 * @param directionNode: node toward which the expansion is going (when relevant)
	 * @return the number of created nodes
	 */
	virtual unsigned connect(Node * fromNode, confPtr_t directionConf, Node * directionNode);

protected:
	//! goal node
	Node * goalNode;
};


/**
 * The Rapidly-exploring Random Tree (RRT) algorithm.
 */
class RRT: public TreePlanner
{
public:
    //! Constructor.
    RRT(Robot * robot, Graph * graph) : TreePlanner(robot, graph) {
        std::cout << "Construct the RRT" << std::endl;
    }

    //! Destructor.
    virtual ~RRT() {};

    /**
     * Initialize the RRT.
     * @return the number of nodes added to the graph
     */
	virtual unsigned init();

	/**
	 * Main function of the RRT.
	 * @return the number of nodes added to the graph
	 */
	virtual unsigned run();

	//! Check whether RRT has found a trajectory.
	virtual bool trajFound();

protected:
	/**
	 * Check the stopping conditions of RRT.
	 * @return TRUE if one of the conditions is satisfied, and FALSE otherwise
	 */
	virtual bool checkStopConditions();

	/**
	 * Perform a single expansion step of RRT, growing the connected component containing fromNode.
	 * @return the number of created nodes
	 */
	unsigned expandOneStep(Node * fromNode);

	/**
	 * Try to link a given node to a given component (accessed through the node compNode).
	 * @return TRUE if the connection is successful.
	 */
	bool connectNodeToComp(Node * node, Node * compNode);

protected:
	//! access point to the expansion procedure of the RRT
	RRTExpansion * expansion;
};

#endif
