
/*
 * TransitionEdge-RRT.cpp
 *
 *  Created on: Sep 17, 2013
 *      Author: bvadant
 */

#include "API/Roadmap/compco.hpp"
#include "API/Roadmap/graph.hpp"

#include "cost_space.hpp"

#include "p3d/proto/p3d_random_proto.h"

#include "planner/Diffusion/Variants/TransitionEdge-RRT.hpp"

using namespace std;


/**
 * ****************************************************
 *      TransitionEdge Expansion
 * ****************************************************
 */

/**
 * Constructor.
 */
TransitionEdgeExpansion::TransitionEdgeExpansion(Graph * graph, Node * goal) : RRTExpansion(graph, goal)
{
    // set the temperature
    temperatureNode = ENV.getDouble(Env::initialTemperature);
    temperatureEdge = ENV.getDouble(Env::initialTemperatureEdge);

    // set the lowest and highest costs
    lowestCostNode = numeric_limits<double>::max();
    highestCostNode = -1. * numeric_limits<double>::max();
    lowestCostEdge = numeric_limits<double>::max();
    highestCostEdge = -1. * numeric_limits<double>::max();

    vector<Node *> nodes = graph->getNodes();
    for (vector<Node *>::iterator n = nodes.begin(); n != nodes.end(); ++n)
        this->updateCostRange((*n)->cost());

    vector<Edge *> edges = graph->getEdges();
    for (vector<Edge *>::iterator e = edges.begin(); e != edges.end(); ++e)
        this->updateCostRangeEdge((*e)->cost());
}



/**
 * Attempt to link two given nodes (i.e. two components, when relevant).
 * @return TRUE if the junction is successful.
 */
bool TransitionEdgeExpansion::attemptJunction(Node * fromNode, Node * toNode)
{
    confPtr_t fromConf = fromNode->getConfiguration();
    confPtr_t toConf = toNode->getConfiguration();

    // path along which the junction is attempted
    LocalPath extensionPath(fromConf, toConf);
    double pathLength = extensionPath.getParamMax();

    // N.B.: the connection is attempted only if the path length is less than a specified threshold
    // (which is some multiple of the extension step)
    if (pathLength <= ENV.getDouble(Env::minimalFinalExpansionGap) * this->step())
    {

        // check cost, and then collisions
        if (ENV.getBool(Env::costBeforeColl))
        {
            // perform the transition test
            if (!this->transitionTest(fromConf, toConf))
                return false;

            // check whether the extension path is valid (i.e. without collision)
            if (!extensionPath.isValid())
                return false;
        }
        // OR check collisions, and then cost
        else
        {
            // check whether the extension path is valid (i.e. without collision)
            if (!extensionPath.isValid())
                return false;

            // perform the transition test
            if (!this->transitionTest(fromConf, toConf))
                return false;
        }

        return true;
    }

    return false;
}



/**
 * TE-RRT Extend.
 * @param fromNode: node from which the expansion is performed
 * @param directionConf: configuration toward which the expansion is going
 * @param directionNode: node toward which the expansion is going (when relevant)
 * @return the number of created nodes
 */
unsigned TransitionEdgeExpansion::extend(Node * fromNode, confPtr_t directionConf, Node * directionNode)
{
    confPtr_t fromConf = fromNode->getConfiguration();
    ConnectedComponent * fromComp = fromNode->getConnectedComponent();

    // path along which the extension is performed
    LocalPath directionPath(fromConf, directionConf);
    double k = this->computeInterpolationFactor(directionPath.getParamMax());

    // Refinement control:
    // (k == 1. && !directionNode) means that we are potentially about to add a new refinement node;
    // in that case, if the number of refinement nodes already present in the component is too high
    // (i.e. more than 10% of the total number of nodes), the expansion attempt is canceled
    if (ENV.getBool(Env::refinementControl) && (k == 1. && !directionNode) &&
            (fromComp->getNumberOfRefinementNodes() > 0.1 * fromComp->getNumberOfNodes()))
        return 0;

    // construct an 'extension path', by linear interpolation along the 'direction path'
    LocalPath extensionPath(fromConf, this->interpolate(directionConf, &directionPath, k));
    confPtr_t newConf = extensionPath.getEnd();

    // check cost, and then collisions
    if (ENV.getBool(Env::costBeforeColl))
    {
        // perform the transition test
        if (!this->transitionTest(fromConf, newConf))
            return 0;

        // check whether the extension path is valid (i.e. without collision)
        if (!extensionPath.isValid())
            return 0;
    }
    // OR check collisions, and then cost
    else
    {
        // check whether the extension path is valid (i.e. without collision)
        if (!extensionPath.isValid())
            return 0;

        // perform the transition test
        if (!this->transitionTest(fromConf, newConf))
            return 0;
    }

    // if everything succeeds, add a new node to the graph
    unsigned nbCreatedNodes(0);
    this->addNode(fromNode, extensionPath, k, directionNode, nbCreatedNodes);

    if (nbCreatedNodes > 0)
    {
        // update the cost range, based on the cost of the new node
        this->updateCostRange(newConf->cost());

        // Refinement control:
        // if a new refinement node has been built, update the number of refinement nodes of the component
        if (ENV.getBool(Env::refinementControl) && (k == 1.))
            fromComp->updateNumberOfRefinementNodes(nbCreatedNodes);
    }

    return nbCreatedNodes;
}


/**
 * TE-RRT Connect.
 * @param fromNode: node from which the expansion is performed
 * @param directionConf: configuration toward which the expansion is going
 * @param directionNode: node toward which the expansion is going (when relevant)
 * @return the number of created nodes
 */
unsigned TransitionEdgeExpansion::connect(Node * fromNode, confPtr_t directionConf, Node * directionNode)
{
    confPtr_t fromConf = fromNode->getConfiguration();
    ConnectedComponent * fromComp = fromNode->getConnectedComponent();

    // path along which the connection is performed
    LocalPath directionPath(fromConf, directionConf);
    double k = this->computeInterpolationFactor(directionPath.getParamMax());

    // Refinement control:
    // (k == 1. && !directionNode) means that we are potentially about to add a new refinement node;
    // in that case, if the number of refinement nodes already present in the component is too high
    // (i.e. more than 10% of the total number of nodes), the expansion attempt is canceled
    if (ENV.getBool(Env::refinementControl) && (k == 1. && !directionNode) &&
            (fromComp->getNumberOfRefinementNodes() > 0.1 * fromComp->getNumberOfNodes()))
        return 0;

    bool success(true);
    confPtr_t currentConf = fromConf;
    confPtr_t newConf;
    double prop(k);
    double oldProp(0.);

    // attempt the connection step by step
    while (success && prop <= 1.)
    {
        // check whether the current extension path is valid (i.e. without collision)
        newConf = this->interpolate(directionConf, &directionPath, prop);
        LocalPath extensionPath(currentConf, newConf);
        if (!extensionPath.isValid())
            success = false;

        // perform the transition test
        if (success && this->transitionTest(currentConf, newConf))
        {
            currentConf = newConf;
            oldProp = prop;
            if (prop < 1.)
                prop = min(1., prop + k);
            else
                prop = 2.;
        }
        else
            success = false;
    }

    // if "reached" or "advanced", add a node to the graph
    if (success || prop > k)
    {
        unsigned nbCreatedNodes(0);
        LocalPath connectPath(fromConf, currentConf);
        this->addNode(fromNode, connectPath, oldProp, directionNode, nbCreatedNodes);

        if (nbCreatedNodes > 0)
        {
            // update the cost range, based on the cost of the new node
            this->updateCostRange(currentConf->cost());

            // Refinement control:
            // if a refinement node has been built, update the number of refinement nodes of the component
            if (ENV.getBool(Env::refinementControl) && (k == 1.))
                fromComp->updateNumberOfRefinementNodes(nbCreatedNodes);
        }

        return nbCreatedNodes;
    }
    else
        return 0;
}

/**
 * Transition test between fromConf and toConf, with temperature tuning.
 * @return TRUE if the transition is accepted
 */
bool TransitionEdgeExpansion::transitionTest(confPtr_t fromConf, confPtr_t toConf)
{
    if(this->transitionTestConf(fromConf,toConf))
        return this->transitionTestEdge(fromConf,toConf);

    return false;
}

bool TransitionEdgeExpansion::transitionTestConf(confPtr_t fromConf, confPtr_t toConf)
{
    double fromCost = fromConf->cost();
    double toCost = toConf->cost();

    // failure, when cost is higher than maximal value
    if ((ENV.getDouble(Env::costMax) > 0.) && (toCost > ENV.getDouble(Env::costMax)))
    {
        //cout<<"Node too high"<<endl;
        return false;
    }
    // success, for down-hill or flat slopes
    if (toCost <= fromCost)
        return true;

    // Metropolis test
    if (exp((fromCost - toCost) / temperatureNode) > 0.5)
    {
        // if success, decrease the temperature
        double costRange = max(1., highestCostNode - lowestCostNode);
        temperatureNode /= pow(2, (toCost - fromCost) / (0.1 * costRange));
        if (temperatureNode < ENV.getDouble(Env::initialTemperature))
            temperatureNode = ENV.getDouble(Env::initialTemperature);
        return true;
    }
    else
    {
        // if failure, increase the temperature
        temperatureNode *= pow(2, ENV.getDouble(Env::temperatureRate));
        //cout<<"Node Metropolis Fail"<<endl;
        return false;
    }
}




bool TransitionEdgeExpansion::transitionTestEdge(confPtr_t fromConf, confPtr_t toConf)
{

    double toCost = global_costSpace->edgeCost(fromConf,toConf);
    double fromCost = toCost;

    vector<Edge*> fromEdges = this->getGraph()->getEdges(fromConf);

    if(fromEdges.size()>0)
    {
        for(vector<Edge*>::iterator it=fromEdges.begin();it!=fromEdges.end();++it)
        {
            double curCost = (*it)->cost();
            if(fromCost < curCost)
                fromCost = curCost;
        }
        cout<<fromCost<<" -> "<<toCost<<endl;
    }
    // failure, when cost is higher than maximal value
    if ((ENV.getDouble(Env::costMaxEdge) > 0.) && (toCost > ENV.getDouble(Env::costMaxEdge)))
    {
        cout<<"Edge too high"<<endl;
        return false;
    }
    // success, for down-hill or flat slopes
    if (toCost <= fromCost)
        return true;

    // Metropolis test
    if (exp((fromCost - toCost) / temperatureEdge) > 0.5)
    {
        // if success, decrease the temperature
        double costRange = max(1., highestCostEdge - lowestCostEdge);
        temperatureEdge /= pow(2, (toCost - fromCost) / (0.1 * costRange));
        if (temperatureEdge < ENV.getDouble(Env::initialTemperatureEdge))
            temperatureEdge = ENV.getDouble(Env::initialTemperatureEdge);
        return true;
    }
    else
    {
        // if failure, increase the temperature
        temperatureEdge *= pow(2, ENV.getDouble(Env::temperatureRateEdge));
        cout<<"Edge Metropolis Fail"<<endl;
        return false;
    }
}




/**
 * ****************************************************
 *      Transition RRT
 * ****************************************************
 */

/**
 * Initialize the T-RRT.
 * @return the number of nodes added to the graph
 */
unsigned TransitionEdgeRRT::init()
{
    // create the T-RRT expansion
    expansion = new TransitionEdgeExpansion(_Graph, _Goal);

    // initialize the Tree Planner
    return TreePlanner::init();
}

/**
 * Perform a single expansion step of RRT, growing the connected component containing fromNode.
 * @return the number of created nodes
 */
unsigned TransitionEdgeRRT::expandOneStep(Node * fromNode)
{
    // sample the expansion direction (i.e. the configuration toward which the expansion will be performed)
    // N.B.: if the expansion is performed toward a node than already exists (e.g. the goal node),
    //       this node becomes the dirNode and dirConf simply points to its configuration
    Node * dirNode(NULL);
    confPtr_t dirConf = expansion->sampleExpansionDirection(dirNode,fromNode);

    // select, from the component to be grown, the node from which the expansion will be attempted
    Node * expandNode = fromNode->getConnectedComponent()->nearestNeighbour(dirConf);

    // directed expansion
    return expansion->expandProcess(expandNode, dirConf, dirNode, ENV.getExpansionMethod());
}
