/*
 *  SimpleReplanner.cpp
 *  libmove3D-planners 
 *
 *  Created by Benjamin Vadant on 01/02/13.
 *  Based on Jim Mainprice replanningAlgorithms.hpp/cpp
 *  Copyright 2013 LAAS/CNRS. All rights reserved.
 *
 */

#include "SimpleReplanner.hpp"
#include "ReplanningSimulator.hpp"

#include "API/project.hpp"

#include "planner/planEnvironment.hpp"
#include "planner/plannerFunctions.hpp"
#include "planner/TrajectoryOptim/Stomp/stompOptimizer.hpp"
#include "planner/TrajectoryOptim/trajectoryOptim.hpp"
#include "planner/TrajectoryOptim/Classic/costOptimization.hpp"
#include "planner/cost_space.hpp"
#include "planner/Diffusion/Variants/Star-RRT.hpp"

#include "HRICS_costspace.hpp"

#include "LightPlanner-pkg.h"
#include "Graphic-pkg.h"
#include "Collision-pkg.h"

#include "move3d-headless.h"

#include <boost/thread/thread.hpp>
#include <sys/time.h>

using namespace std;
using namespace tr1;

SimpleReplanner::SimpleReplanner(Robot* r) : Replanner(r)
{
    
}

SimpleReplanner::~SimpleReplanner()
{
    
}

//! Before simple (no softmotion) replanning execution, this function initializes the replanner data
bool SimpleReplanner::init()
{
    isPlanning = false;

    setActiveJoints( ENV.getInt(Env::setOfActiveJoints) );
    initActiveJoints();
    
    // Store initial and final configurations
    qSwitch = robot->getCurrentPos();
    qGoal   = robot->getGoTo();
    
    idRun = 0;
    
    switch (PlanEnv->getInt(PlanParam::replanningInitMethod)) 
    {
        case 0:
            if( !initCreateStraightline() ) {
                cout << "Error : could not create straightline" << endl;
                return false;
            }
            break;
            
        case 1:
            if( !p3d_run_rrt( robot->getRobotStruct() ) ){
                cout << " Error : could initialize with RRT" << endl;
                return false;
            }
            else {
                currentTrajectory = p3d_get_last_trajectory();
                cout << "currentTrajectory.getRangeMax() : " << currentTrajectory.getRangeMax() << endl;
            }
            break;
            
        default:
            cout << "No initializiation" << endl;
            return false;
    }
    traj_optim_initStomp();
    globalRePlanningEnv->setDrawStep( currentTrajectory.getRangeMax()/30 );                            //simulator
    globalRePlanningEnv->storeTrajectoryToDraw( currentTrajectory, 0 );                                   //simulator
    return true;
}

//! Creates a straight line trajectory
//! uses the main robot of the simulator or uses the current
//! robot if not initialized
bool SimpleReplanner::initCreateStraightline()
{
    
    if( robot == NULL ) {
        robot = global_Project->getActiveScene()->getActiveRobot();
    }
    
    if (!robot) {
        cout << "Error : robot not initialized in file " << __FILE__ << " ,  " << __func__ << endl;
        return false;
    }
    
    confPtr_t qInit( robot->getInitialPosition() );
    confPtr_t qGoal( robot->getGoTo() );
    
    if( qInit->equal( *qGoal ) )
    {
        cout << "Error : qInit == qGoal in file " << __FILE__ << " ,  " << __func__ << endl; 
        cout << "for robot : " << robot->getName() << endl;
        qInit->print();
        qGoal->print();
        return false;
    }
    
    vector<confPtr_t> configurations(2);
    
    configurations[0] = qInit;
    configurations[1] = qGoal;
    
    currentTrajectory = API::Trajectory( configurations );
    currentTrajectory.replaceP3dTraj();
    return true;
}

//! Set the DoFs active/passive of the robot for navigation
void SimpleReplanner::initForNavigation()
{  
    if( useMultiLocalPath )
    {
        cout << "Set base MultiLocalPath for Navigation " << endl;
        
#ifdef MULTILOCALPATH
        p3d_multiLocalPath_disable_all_groupToPlan( robot->getRobotStruct() , false );
        p3d_multiLocalPath_set_groupToPlan( robot->getRobotStruct(), baseMultiLocalPath, 1, false);
#endif
        
        // Fix all joints for planner
        // This only works with manipulation planner
        fixAllJointsExceptBase( robot->getRobotStruct() );
    }
    
    // Set base to joint to draw
    p3d_set_user_drawnjnt(1);
}

void SimpleReplanner::initForManipulation()
{
    if( useMultiLocalPath )
    {
        cout << "Set upbody MultiLocalPath for Manipulation" << endl;
        
#ifdef MULTILOCALPATH
        p3d_multiLocalPath_disable_all_groupToPlan( robot->getRobotStruct() , false );
        p3d_multiLocalPath_set_groupToPlan( robot->getRobotStruct(), upBodyMultiLocalPath, 1, false);
#endif

        // Fix all the joints but unfixes the arm 0
        fixAllJointsWithoutArm( robot->getRobotStruct(), 0 );
    }
}

void SimpleReplanner::initForMobileManipulation()
{
    if( useMultiLocalPath )
    {
        cout << "Set upbody MultiLocalPath for Mobile Manipulation" << endl;
        
#ifdef MULTILOCALPATH
        p3d_multiLocalPath_disable_all_groupToPlan( robot->getRobotStruct() , false );
        p3d_multiLocalPath_set_groupToPlan( robot->getRobotStruct(), upBodyMultiLocalPath, 1, false);
#endif
    }
    
    // Fix all the joints but unfixes the arm 0
    fixAllJointsWithoutArm( robot->getRobotStruct(), 0 );
    
    // Unfix base joint for planning
    unFixJoint( robot->getRobotStruct(), robot->getRobotStruct()->baseJnt );
    
}

//! Choose what part of the robot are active in the planning phase
//! Three type of planning to chose from
//! Navigation, Manipulation, Mobile-anipulation
void SimpleReplanner::setActiveJoints(int type)
{
    switch (type) 
    {
        case 0:
            replanningType = NAVIGATION;
            break;
            
        case 1:
            replanningType = MANIPULATION;
            break;
            
        case 2:
            replanningType = MOBILE_MANIP;
            break;
            
        default:
            cout << "Planner type not implemented defaulting to NAVIGATION" << endl;
            replanningType = NAVIGATION;
            break;
    }
}

void SimpleReplanner::initActiveJoints()
{
    // Block specific joint sampling
    // also activate localpaths
    switch (replanningType) 
    {
        case NAVIGATION:
            cout << "Init for Navigation" << endl;
            initForNavigation();
            break;
            
        case MANIPULATION:
            cout << "Init for Manipulation" << endl;
            initForManipulation();
            break;
            
        case MOBILE_MANIP:
            cout << "Init for Mobile-manipulation" << endl;
            initForMobileManipulation();
            break;
    }
}

void SimpleReplanner::run()
{
    isPlanning = true;
    cout << "SimpleReplanner :: Start replanning step : " << __func__ << endl;
    if(!optimizer->humanHasMoved())
    {
        cout << "SimpleReplanner :: human has not moved => no replanning"<<endl;
        planningSucceeded = true;
    }
    else
    {
        planningSucceeded=false;

        PlanEnv->setBool(   PlanParam::planWithTimeLimit, true );
        PlanEnv->setBool(   PlanParam::trajWithTimeLimit, true );
        PlanEnv->setDouble( PlanParam::timeLimitSmoothing, timeLimitRePlanning-0.3 );// Set 0.3 seconds to do the concatenation of trajectories
        cout << "timeLimitRePlanning : " << timeLimitRePlanning << endl;
        cout << "positionSwitch : " << positionSwitch << endl;
        cout << "currentTrajectory.getRangeMax() : " << currentTrajectory.getRangeMax() << endl;

        idRun++;

        API::Trajectory newPortion = currentTrajectory.extractSubTrajectory( positionSwitch, currentTrajectory.getRangeMax(), false);
        cout << "newPortion.getNbOfPaths() : " << newPortion.getNbOfPaths() << endl;
        cout << "newPortion.getRangeMax() : " << newPortion.getRangeMax() << endl;

        if( newPortion.getNbOfPaths() != 0 )
        {
            API::CostOptimization costOptimization( newPortion );
            costOptimization.setStep( initialStep/PlanEnv->getDouble(PlanParam::PlanParam::MaxFactor) );
            costOptimization.runDeformation( PlanEnv->getInt(PlanParam::smoothMaxIterations), idRun++ );

            std::pair<bool,API::Trajectory> concatenatedTrajectory = concatenateToCurrentTrajectory( costOptimization );
            planningSucceeded = concatenatedTrajectory.first;

            if( planningSucceeded ) {
                currentTrajectory = concatenatedTrajectory.second;
            }
            else {
                if( costOptimization.getBegin() != qSwitch ) {
                    cout << "New portion begin != qSwitch" << endl;
                }
            }
        }
        else
        {
            cout << "newPortion.getNbOfPaths() : 0" << endl;
        }

        if( !planningSucceeded ) {
            cout << "SimpleReplanner :: Replanning failed" << endl;
        }
    }
    
    cout << "SimpleReplanner :: End replanning step : " << __func__ << endl;
    isPlanning = false;
}
