/*
 *  SoftmotionReplanner.cpp
 *  libmove3D-planners 
 *
 *  Created by Benjamin Vadant on 01/02/13.
 *  Based on Jim Mainprice replanningAlgorithms.hpp/cpp
 *  Copyright 2013 LAAS/CNRS. All rights reserved.
 *
 */

#include "SoftmotionReplanner.hpp"
#include "ReplanningSimulator.hpp"

#include "API/project.hpp"

#include "planner/planEnvironment.hpp"
#include "planner/plannerFunctions.hpp"
#include "planner/TrajectoryOptim/Stomp/stompOptimizer.hpp"
#include "planner/TrajectoryOptim/trajectoryOptim.hpp"
#include "planner/TrajectoryOptim/Classic/costOptimization.hpp"
#include "planner/cost_space.hpp"
#include "planner/Diffusion/Variants/Star-RRT.hpp"

#include "HRICS_costspace.hpp"

#include "LightPlanner-pkg.h"
#include "Graphic-pkg.h"
#include "Collision-pkg.h"

#include "move3d-headless.h"

#include <boost/thread/thread.hpp>
#include <sys/time.h>

using namespace std;
using namespace tr1;

SoftmotionReplanner::SoftmotionReplanner(Robot* r) : Replanner(r)
{

}

SoftmotionReplanner::~SoftmotionReplanner()
{
    
}

bool SoftmotionReplanner::init()
{
    if( robot == NULL )
    {
        cout << "Error: robot is not initialized in " << __func__ << endl;
        return false;
    }
    
    manipulationPlanner = new ManipulationPlanner(robot->getRobotStruct());
    manipulationPlanner->setUseBaseMotion( true );
    manipulationPlanner->setPlanningMethod( p3d_planner_function );
    manipulationPlanner->setSmoothingMethod( p3d_smoothing_function );
    manipulationPlanner->setCleanningRoadmaps( false );
    return true;
}

//! Convert trajectory to softmotion
//! @return the result in SM_TRAJ form
//! @return true or false is succeded
bool SoftmotionReplanner::computeSoftmotion(MANPIPULATION_TRAJECTORY_CONF_STR &confs, p3d_traj* traj, SM_TRAJ& smTraj)
{
    bool approximate = false;
    if (!traj) {
        cout << "SoftMotion : ERREUR : no trajectory has been given"<<endl;
        return false;
    }
    if (traj->nlp < 1) {
        cout << "SoftMotion : Optimization is not possible: current trajectory contains zero local path"<<endl;
        return false;
    }
    if (p3d_local_get_planner() != MULTI_LOCALPATH) {
        cout << "SoftMotion : Optimization is not possible: current trajectory is not a multi-localpath one"<<endl;
        return false;
    }
    
    for (int i = 0; i < int((*traj->rob->armManipulationData).size()); i++) {
        ArmManipulationData& armData  = (*traj->rob->armManipulationData)[i];
        if (armData.getCartesian()) {
            approximate = true;
        }
    }
    
    if (p3d_convert_traj_to_softMotion(traj, ENV.getBool(Env::smoothSoftMotionTraj), true, 
                                       approximate, confs.first, confs.second, smTraj) == 1) {
        cout << "SoftMotion : p3d_optim_traj_softMotion : cannot compute the softMotion trajectory"<<endl;
        return false;
    }
    
    return true;
}

//! Comutes softmotion on a linear trajectory
//! @param traj the linear trajectory that has going to be converted
//! @return set the result in globalManipPlannerLastTraj
bool SoftmotionReplanner::computeSoftmotion(p3d_traj* traj)
{
    if (traj != NULL)
    {
        MANPIPULATION_TRAJECTORY_CONF_STR conf;
        SM_TRAJ smTraj;
        computeSoftmotion(conf, traj, smTraj);
        globalManipPlannerLastTraj = smTraj;                                          //simulator
        return true;
    }
    else
    {
        cout << "SoftMotion : ERREUR : no trajectory has been given"<<endl;
        return false;
    }
}

//DEPRECATED
//! Generates the CurrentTraj and the soft motion traj
bool SoftmotionReplanner::generateNewTrajectory(const vector<p3d_traj*>& trajectories)
{
    if( planningSucceeded )
    {
        p3d_traj* entireTraj = concatenateToCurrentTrajectory( trajectories );
        
        if ( entireTraj == NULL ) 
        {
            cout << "Fail to concat traj" << endl;
            planningSucceeded = false;
        }
        else if( !computeSoftmotion(entireTraj) )
        {
            cout << "Fail to compute softmotion" << endl;
            planningSucceeded = false;
        }
    }
    
    if( planningSucceeded )
    {
        p3d_multiLocalPath_disable_all_groupToPlan( robot->getRobotStruct(), false );
        p3d_multiLocalPath_set_groupToPlan( robot->getRobotStruct(), upBodyMultiLocalPath, 1, false );
        
        p3d_traj* lin_traj =  p3d_get_last_linear_traj();
        
        if( lin_traj )
        {
            currentTrajectory = API::Trajectory( robot, lin_traj );
        }
        else
        {
            cout << "Failed to get linear traj" << endl;
            planningSucceeded = false;
        }
    }
    
    return planningSucceeded;
}

//! Replan a trajectory for the entire robot using the manipulation planner
//! the trajectory is planned using RRT and a soft motion trajectory is the produced
void SoftmotionReplanner::run()
{
    isPlanning = true;
    cout << "SoftmotionReplanner :: Start replanning step : " << __func__ << endl;
    
    planningSucceeded=false;

    if ( robot == NULL || manipulationPlanner == NULL ) 
    {
        cout << "SoftmotionReplanner :: Error : replanning not well initialized in " << __func__ << __FILE__ << endl;
        return;
    }
    
    idRun++;
    
    MANIPULATION_TASK_TYPE_STR type = ARM_FREE;
    MANIPULATION_TASK_MESSAGE status = MANIPULATION_TASK_NO_TRAJ_FOUND;
    vector <p3d_traj*> trajectories;
    string m_OBJECT_NAME;
    vector<double> objStart, objGoto;
    objGoto.resize(6,P3D_HUGE);
    
    configPt qInit  = qSwitch->getConfigStructCopy();
    configPt qGoal  = this->qGoal->getConfigStructCopy();
    
    string str = manipulationPlanner->robot()->name;
    
    cout << "Planning for robot : " << str << endl;
    
    if( str == "PR2_ROBOT" || str == "BOB_ROBOT"  || str == "MAX_ROBOT" )
    {
        fixAllJointsWithoutArm(manipulationPlanner->robot(),0);
    }
    
    switch ( (unsigned int) manipulationPlanner->robot()->lpl_type ) 
    {			
        case MULTI_LOCALPATH : 
           {
            cout << "armPlanTask :: MultiLocalPath" << endl;
            gpGrasp grasp;
            trajectories.clear();
            status = manipulationPlanner->armPlanTask(type, 0, qInit, qGoal, objStart, objGoto,  
                                                 "", "", "", grasp, trajectories);
            break;
           }
        case LINEAR :
            cout << "Manipulation : linear not implemented" << endl;
            break;
            
        case SOFT_MOTION : 
            cout << "Manipulation : softmotion not implemented" << endl;
            break;
    }
    
    planningSucceeded = ( status == MANIPULATION_TASK_OK );
    planningSucceeded = ( generateNewTrajectory(trajectories) );

    cout << "SoftmotionReplanner :: End replanning step : " << __func__ << endl;
    isPlanning = false;
}
