#include "cost_space.hpp"
#include <iostream>

#include "API/project.hpp"
#include "API/Roadmap/compco.hpp"

#include "planner/TrajectoryOptim/Stomp/stompOptimizer.hpp"
#include "planEnvironment.hpp"

#include "P3d-pkg.h"
#include "Localpath-pkg.h"
#include "GroundHeight-pkg.h"
#include "Planner-pkg.h"

#include "Collision-pkg.h"

#include "lightPlanner/proto/ManipulationArmData.hpp"

#include <algorithm>

#include <boost/function.hpp>
#include <boost/bind.hpp>

extern void* GroundCostObj;

using namespace std;
using namespace tr1;

//------------------------------------------------------------------------------

CostSpace* global_costSpace(NULL);


void GlobalCostSpace::initialize()
{
    if (global_costSpace)
        return;

    global_costSpace = new CostSpace();

    global_costSpace->addCost("NoCost", boost::bind(computeFlatCost, _1));
    global_costSpace->addCost("costDistToObst", boost::bind(computeDistanceToObstacles, _1));
    global_costSpace->addCost("costClearance", boost::bind(computeClearance, _1));
    global_costSpace->addCost("costFlyCrane", boost::bind(computeFlyCranecost, _1));
    global_costSpace->addCost("costIsInCollision", boost::bind(computeInCollisionCost, _1));
    global_costSpace->addCost("costHot", boost::bind(computeHotCost, _1));

    global_costSpace->setCost("NoCost");

    if (GroundCostObj) {
        global_costSpace->addCost("costMap2D", boost::bind(computeIntersectionWithGround, _1));
        global_costSpace->setCost("costMap2D");
    }
}

//------------------------------------------------------------------------------
CostSpace::CostSpace() : mSelectedCostName("No Cost"), m_deltaMethod(cs_integral), m_resolution(cs_classic)/* m_resolution(cs_pr2_manip)*/ 
{
  m_dmax = p3d_get_env_dmax();
  
  //m_resolution = cs_pr2_manip;
}

//------------------------------------------------------------------------------
std::string CostSpace::getSelectedCostName()
{
  return mSelectedCostName;
}

//------------------------------------------------------------------------------
std::vector<string> CostSpace::getAllCost()
{
	std::vector<string> functions;
	std::map< string, boost::function<double(Configuration&)> >::iterator it;
	
	for ( it=mFunctions.begin() ; it != mFunctions.end(); it++ )
		functions.push_back((*it).first);
	
	return functions;
}

//------------------------------------------------------------------------------
bool CostSpace::setCost(string name)
{
	if(mFunctions.find(name) != mFunctions.end())
	{
    mSelectedCostName = name;
		mSelectedCost = mFunctions[name];
    return true;
	}
	else
	{
		std::cout << "Warning : in CostSpace::setCost(string name), could not find a cost function named " << name  << std::endl;
    return false;
	}
}
//------------------------------------------------------------------------------
void CostSpace::addCost(string name, 
												boost::function<double(Configuration&)> f)
{
	if(mFunctions.find(name) == mFunctions.end())
	{
		mFunctions[name] = f;
	}
	else
	{
		std::cout << "Warning : in CostSpace::addCost, replacing the cost function named " << name << "by another." << std::endl;
		mFunctions[name] = f;
	}
}
//------------------------------------------------------------------------------
void CostSpace::deleteCost(string name)
{
  std::map< string, boost::function<double(Configuration&)> >::iterator 
  it = mFunctions.find(name);
	
  if(it != mFunctions.end())
	{
		mFunctions.erase (it);
	}
	else 
	{
		std::cout << "Warning : in CostSpace::deleteCost, the cost function " << name << " does not exist." << std::endl;
	}
}

//------------------------------------------------------------------------------
double CostSpace::cost(Configuration& conf)
{
  shared_ptr<Configuration> q_tmp = conf.copy();
  
	if(!mSelectedCost.empty())
	{
		double cost = mSelectedCost(conf);
    
    if( !conf.equal( *q_tmp ) )
    {
      cout << "Cost function modifies the config" << endl;
    }
    return cost;
	}
	else
	{
		std::cout << "Warning : CostSpace::cost(Configuration& conf) called, but \
		the cost function has not been set." << std::endl;
		return(1.0);
	}
}

//------------------------------------------------------------------------------
/**
 * ComputeDeltaStepCost
 * Compute the cost of a portion of path */
double CostSpace::deltaStepCost(double cost1, double cost2, double length)
{
	const double epsilon = 0.002;
	//double alpha;
	const double kb = 0.00831;
  const double temp = 310.15;
	
	if ( ENV.getBool(Env::isCostSpace) )
	{
		switch (m_deltaMethod)
		{
			case cs_mechanical_work:
      {
				double cost;
				if (cost2 > cost1)
				{
					cost = length * epsilon + cost2 - cost1;
				}
				else
				{
					cost = epsilon * length;
				}
				return cost;
      }
				
			case cs_integral:
			case cs_visibility:
      {
        double powerOnIntegral = 1.0;
        double cost = pow(((cost1 + cost2)/2),powerOnIntegral)*length;
        
        // Warning, length added for dynamic shortcut (HRI)
        cost += epsilon*length;
        
        return cost;
      }
				
			case cs_average:
				return (cost1 + cost2) / 2.;
        
      case cs_max:
        return std::max( cost1, cost2 );
				
			case cs_boltzman_cost:
				
				if (cost2 > cost1)
					return 1;
				return 1/exp(ENV.getInt(Env::maxCostOptimFailures)*(cost2-cost1)/(kb*temp));
				
			default:
				std::cout << "Warning: " << __func__ <<  std::endl;
		}
	}
	//no cost function
	return length;
}

double CostSpace::edgeCost(confPtr_t from, confPtr_t to)
{

    double angleToPreviousConfiguration = 0;

    double paramAngleToPrevious = ENV.getDouble(Env::paramAngleToPrevious);
    //Hot face related information computation
    std::vector<ArmManipulationData>* armData = from->getRobot()->getRobotStruct()->armManipulationData;
    p3d_rob* carriedObject = armData->at(0).getCarriedObject();

    poly_plane* hotPlane = carriedObject->ocur->polcur->poly->the_faces[0].plane ;//The first plane defined in the p3d file is the dangerous one (if there is more than one dangerous plane ???)

    //In order to get the normal vector to the plane corresponding to the dangerous face we have to move the orginal normale to the current position of the object
    double x,y,z,cx,sx,cy,sy,cz,sz;

    x = hotPlane->normale[0];
    y = hotPlane->normale[1];
    z = hotPlane->normale[2];

    cx = cos(to->getRobot()->getJoint("virtual_object_right")->getJointDof(3));
    sx = sin(to->getRobot()->getJoint("virtual_object_right")->getJointDof(3));

    cy = cos(to->getRobot()->getJoint("virtual_object_right")->getJointDof(4));
    sy = sin(to->getRobot()->getJoint("virtual_object_right")->getJointDof(4));

    cz = cos(to->getRobot()->getJoint("virtual_object_right")->getJointDof(5));
    sz = sin(to->getRobot()->getJoint("virtual_object_right")->getJointDof(5));

    poly_vector3 hotNormale;

    hotNormale[0] =  x*cy*cz + y*( sx*sy*cz + cx*sz) + z*(-cx*sy*cz + sx*sz);
    hotNormale[1] = -x*cy*sz + y*(-sx*sy*sz + cx*cz) + z*( cx*sy*sz + sx*cz);
    hotNormale[2] =  x*sy    + y*(-sx*cy)            + z*(cx*cy);

    double hotNorm = sqrt(pow(hotNormale[0],2.0)+pow(hotNormale[1],2.0)+pow(hotNormale[2],2.0));

    poly_vector3 fromTovector;

    fromTovector[0] = to->getConfigStruct()[to->getRobot()->getJoint("virtual_object_right")->getIndexOfFirstDof()] - from->getConfigStruct()[from->getRobot()->getJoint("virtual_object_right")->getIndexOfFirstDof()];
    fromTovector[1] = to->getConfigStruct()[to->getRobot()->getJoint("virtual_object_right")->getIndexOfFirstDof() + 1] - from->getConfigStruct()[from->getRobot()->getJoint("virtual_object_right")->getIndexOfFirstDof() + 1];
    fromTovector[2] = to->getConfigStruct()[to->getRobot()->getJoint("virtual_object_right")->getIndexOfFirstDof() + 2] - from->getConfigStruct()[from->getRobot()->getJoint("virtual_object_right")->getIndexOfFirstDof() + 2];

    double fromNorm = sqrt(pow(fromTovector[0],2.0)+pow(fromTovector[1],2.0)+pow(fromTovector[2],2.0));
    double prodsca1 = hotNormale[0]*fromTovector[0] + hotNormale[1]*fromTovector[1] + hotNormale[2]*fromTovector[2];

    angleToPreviousConfiguration = abs((acos(prodsca1/(hotNorm*fromNorm)) * 180.0 / 3.14159265));//palier

    if(angleToPreviousConfiguration < 60 && angleToPreviousConfiguration > 45)
    {    angleToPreviousConfiguration *= 5;}
    else if(angleToPreviousConfiguration < 45)
    {     angleToPreviousConfiguration *= 10; }


    double cost = exp((paramAngleToPrevious*((angleToPreviousConfiguration/180)-1))) - 1;

    return cost;
}



//----------------------------------------------------------------------
void CostSpace::setNodeCost( Node* node, Node* parent )
{
  //------------------------------------------------------
  // New node cost
  //------------------------------------------------------
  if ( parent != NULL )
  {
    confPtr_t q_tmp = parent->getConfiguration()->copy();
    
    // Compute the sum of cost of the node
    // TODO fix task space bug (should remove the copy)
//    if( set_sum_cost ) 
//    {
//      LocalPath path( parent->getConfiguration()->copy(), node->getConfiguration() );
//      node->sumCost() = parent->sumCost() + path.cost();
//    }
    
    // Min and max cost of the compco
    node->getConnectedComponent()->getCompcoStruct()->minCost 
    = std::min(node->getConfiguration()->cost(),
               node->getConnectedComponent()->getCompcoStruct()->minCost );
    
    node->getConnectedComponent()->getCompcoStruct()->maxCost 
    = std::max(node->getConfiguration()->cost(),
               node->getConnectedComponent()->getCompcoStruct()->maxCost );
    
    if ( !parent->getConfiguration()->equal(*q_tmp) )
    {
      cout << "Configuration was modified" << endl;
    }
  }
}


/*if (p3d_GetCostMethodChoice() == URMSON_TRANSITION)
 {
 node->getConnectedComponent()->getCompcoStruct()->maxUrmsonCost
 = MAX(NodePt->sumCost +
 p3d_ComputeUrmsonCostToGoal(_Graph->getGraphStruct(),NodePt) ,
 node->getConnectedComponent()->getCompcoStruct()->maxUrmsonCost);
 }	*/

//jnt->getName() : right-Arm1(6) , index_dof : 16
//jnt->getName() : right-Arm2(7) , index_dof : 17
//jnt->getName() : right-Arm3(8) , index_dof : 18
//jnt->getName() : right-Arm4(9) , index_dof : 19
//jnt->getName() : right-Arm5(10) , index_dof : 20
//jnt->getName() : right-Arm6(11) , index_dof : 21
//jnt->getName() : right-Arm7(12) , index_dof : 22

//jnt->getName() : virtual_object_right(32) , index_dof : 37
//jnt->getName() : virtual_object_right(32) , index_dof : 38
//jnt->getName() : virtual_object_right(32) , index_dof : 39
//jnt->getName() : virtual_object_right(32) , index_dof : 40
//jnt->getName() : virtual_object_right(32) , index_dof : 41
//jnt->getName() : virtual_object_right(32) , index_dof : 42

void CostSpace::getPr2ArmConfiguration( Eigen::VectorXd& x, confPtr_t q )
{ 
  x[0] = (*q)[16];
  x[1] = (*q)[17];
  x[2] = (*q)[18];
  x[3] = (*q)[19];
  x[4] = angle_limit_PI((*q)[20]);
  x[5] = (*q)[21];
  x[6] = angle_limit_PI((*q)[22]);
  
  x[7] = (*q)[37];
  x[8] = (*q)[38];
  x[9] = (*q)[39];
  x[10] = angle_limit_PI((*q)[40]);
  x[11] = angle_limit_PI((*q)[41]);
  x[12] = angle_limit_PI((*q)[42]);
}

double CostSpace::getPr2ArmDistance( Robot* robot, Eigen::VectorXd& q_i, Eigen::VectorXd& q_f )
{
  p3d_jnt** joints = robot->getRobotStruct()->joints;
  const int joints_id[] = { 6,7,8,9,10,11,12,32 };
  double dist=0.0,ljnt=0.0;
  
  int k=0;
  
  printf("\n");
  for (int i=0; i<8; i++) 
  {
    p3d_jnt* jntPt = joints[joints_id[i]];
    
    cout << "jntPt->name : " << jntPt->name << " , " << jntPt->dist << endl;
    
    for (int j=0; j<jntPt->dof_equiv_nbr; j++) 
    {
      if (p3d_jnt_is_dof_angular(jntPt, j)) 
      {
        if (p3d_jnt_is_dof_circular(jntPt, j)) 
          dist = fabs(jntPt->dist*dist_circle(q_i[k], q_f[k]));
        else 
          dist = fabs(jntPt->dist*(q_f[k] - q_i[k]));
      }
      else{
        dist = fabs(q_f[k] - q_i[k]);
      }
      
      if (isnan(dist)) {
        printf("Distance computation error !!!\n");
        return P3D_HUGE;
      }
      
      dist = SQR(dist);
      
      printf(" dist[%d] = %f\n", jntPt->index_dof + j, dist );
      
      ljnt += dist;
      k++;
    }
  }
  return std::sqrt(ljnt);
}

//----------------------------------------------------------------------
double CostSpace::cost(LocalPath& path, int& nb_test)
{
    double cost = 0.0;
    nb_test = 0;

    int nStep;
    double deltaStep;
    const double pathLength = path.getParamMax();
    
    if( m_resolution == cs_classic )
    {
      deltaStep = p3d_get_env_dmax();
      nStep = pathLength / deltaStep; 
    }
    
    if( m_resolution == cs_pr2_manip )
    {
      Eigen::VectorXd a( Eigen::VectorXd::Zero(13) );
      Eigen::VectorXd b( Eigen::VectorXd::Zero(13) );
      
      path.getLocalpathStruct();
      
      getPr2ArmConfiguration( a, path.getBegin() );
      getPr2ArmConfiguration( b, path.getEnd() );
      
//      cout << "a : " << endl << a << endl;
//      cout << "b : " << endl << b << endl;
//      
//      path.getBegin()->print();
//      path.getEnd()->print();
      
//      double param_max = ( a - b ).norm();
      double param_max  = getPr2ArmDistance( path.getRobot(), a, b);
      nStep = ceil(param_max/(PlanEnv->getDouble(PlanParam::costResolution)*m_dmax));
      deltaStep = param_max/nStep;
      
      cout <<  endl;
      cout << "param_max : " << param_max << endl;
      cout << "path.getParamMax() : " << pathLength << endl;
      
//      cout << "param_max : " << param_max << " , nStep : " << nStep << endl;
//      cout << "a : " << endl << a << endl;
//      cout << "b : " << endl << b << endl;
    }
    
	// iterate over the path
	double previousConfCost = path.getBegin()->cost();
	double param = deltaStep;
	while (param <= pathLength)
	{
		// compute the cost of the last portion of the path
		if (pathLength - param < 10e-6)
		{
			if (ENV.getInt(Env::costDeltaMethod) == MECHANICAL_WORK)
				cost += max(0., path.getEnd()->cost() - previousConfCost);
			else if (ENV.getInt(Env::costDeltaMethod) == INTEGRAL)
				cost += deltaStep * (previousConfCost + path.getEnd()->cost()) / 2.;
			else
				cost += 0.;
			break;
		}
		// compute the cost of a regular portion of the path
		else
		{
			double currentConfCost = this->cost(*path.configAtParam(param));

			if (ENV.getInt(Env::costDeltaMethod) == MECHANICAL_WORK)
				cost += max(0., currentConfCost - previousConfCost);
			else if (ENV.getInt(Env::costDeltaMethod) == INTEGRAL)
				cost += deltaStep * (previousConfCost + currentConfCost) / 2.;
			else
				cost += 0.;

			// go to next portion
			previousConfCost = currentConfCost;
			if (pathLength - param > deltaStep)
				param += deltaStep;
			else {
				deltaStep = pathLength - param;
				param = pathLength;
			}
		}
	}
    
    nb_test = nStep;

	return cost;
}

/*************************************************************************************
 ****************        FlyCrane      ***********************************************
 *************************************************************************************/

/**
 * Initialize the FlyCrane cost space.
 */
void CostSpace::initializeFlyCraneCostSpace()
{
    // TODO DD Test the second metric
    // set the distance associated with this C-space
    // ENV.setInt(Env::DistConfigChoice, FLYCRANE_CSPACE);

    // coordinates of the platform anchor points expressed in the local reference frame
    B[0] << -.433,   .15, 0.;
    B[1] << -.433,  -.15, 0.;
    B[2] <<  .0866, -.45, 0.;
    B[3] <<  .3464, -.3,  0.;
    B[4] <<  .3464,  .3,  0.;
    B[5] <<  .0866,  .45, 0.;

    // length of the cables
    for (unsigned i(0); i < 6; ++i)
        cableLength[i] = 1.2;

    // interval of admissible tension for each cable
    for (unsigned i(0); i < 6; ++i) {
        admissibleTension[i][0] = 0.;
        admissibleTension[i][1] = 6.64;
    }

    // wrench exerted on the platform
    wrench << 0., 0., -2.45, 0., 0., 0. ;

    // coordinates (in the local reference frame) of the platform point on which the wrench is exerted
    wrenchPoint <<  0., 0., 0. ;
    // TODO DD: Make this a parameter.
    // puzzle.p3d
//    wrenchPoint <<  .0173, .35, -.777 ;

    // ellipsoid defining the uncertainty region around the given wrench
    double radius[6] = { 0.04, 0.04, 0.04, 0.04, 0.04, 0.04 };
    Ellipsoid.setIdentity();
    for (unsigned i(0); i < 6; ++i)
        Ellipsoid(i, i) = 1. / (radius[i] * radius[i]);

    // mass of the quadrotors
    for (unsigned i(0); i < 3; ++i)
        quadMass[i] = 0.477;

    // maximal thrust exerted by the quadrotors
    for (unsigned i(0); i < 3; ++i)
    	quadThrust[i] = 6.64;

    // offset of the anchor point wrt the center of mass of each quadrotor
    for (unsigned i(0); i < 3; ++i)
    	offsetAnchorCM[i] = 0.035;

    // distance from the center of mass to the center of the motors
    for (unsigned i(0); i < 3; ++i)
    	dCMmotor[i] = 0.17;

    // compute the weight of each quadrotor
    quadWeight[0] << 0, 0, -quadMass[0] * 9.81;
    quadWeight[1] << 0, 0, -quadMass[1] * 9.81;
    quadWeight[2] << 0, 0, -quadMass[2] * 9.81;

    // compute the vectors formed by the platform anchor points, and their norms
    Eigen::Vector3d vBaux[3];
    vBaux[0] = B[5] - B[0];
    vBaux[1] = B[1] - B[2];
    vBaux[2] = B[3] - B[4];
    double nB[3];
    for (unsigned i(0); i < 3; ++i)
    	nB[i] = vBaux[i].norm();

    // normalize these vectors
    for (unsigned i(0); i < 3; ++i)
    	vB[i] = vBaux[i].normalized();

    // compute the angles formed by each pair of vector vBij and cable i
    double * L = cableLength;
    beta[0] = acos((nB[0] * nB[0] + L[0] * L[0] - L[5] * L[5]) / (2 * nB[0] * L[0]));
    beta[1] = acos((nB[1] * nB[1] + L[2] * L[2] - L[1] * L[1]) / (2 * nB[1] * L[2]));
    beta[2] = acos((nB[2] * nB[2] + L[4] * L[4] - L[3] * L[3]) / (2 * nB[2] * L[4]));

    // test the validity of the anchor points
    Eigen::Matrix3d vBij;
    vBij << vB[0], vB[1], vB[2];
    Eigen::FullPivLU<Eigen::Matrix3d> lu_decomp(vBij);
    if (lu_decomp.rank() != 2)
        cout << "*** WARNING FlyCrane: platform anchor points are aligned or not coplanar !!! ***" << endl;
}


/**
 * Compute the rotation matrix of a given vector of angles.
 * (This function is useful mainly to keep track of the Euler convention used here.)
 */
Eigen::Matrix3d CostSpace::RotationMatrix(double orientation[])
{
    Eigen::Matrix3d Rot;
    Rot = Eigen::AngleAxisd(orientation[0], Eigen::Vector3d::UnitX()) *
          Eigen::AngleAxisd(orientation[1], Eigen::Vector3d::UnitY()) *
          Eigen::AngleAxisd(orientation[2], Eigen::Vector3d::UnitZ());
    return Rot;
}


/**
 * Compute the coordinates (in the global reference frame) of the anchor points, Aij, of the quadrotors
 * and fill the corresponding attributes in the given configuration.
 */
void CostSpace::computeQuadAnchorPoints(Configuration * conf)
{
    Eigen::Matrix3d RotZ, RotAxisB;

    // get the position, orientation (as rotation matrix) and dihedral angles stored in the configuration
    Eigen::Vector3d P = conf->getPosition();
    Eigen::Matrix3d Rot = this->RotationMatrix(conf->getOrientation());
    double * D = conf->getDihedralAngles();

    Eigen::Vector3d A[3];

    // compute A05
    RotZ = Eigen::AngleAxisd(beta[0], Eigen::Vector3d::UnitZ());
    RotAxisB = Eigen::AngleAxisd(M_PI_2 - D[0], vB[0]);
    A[0] = P + Rot * B[0] + cableLength[0] * Rot * RotAxisB * RotZ * vB[0];

    // compute A21
    RotZ = Eigen::AngleAxisd(beta[1], Eigen::Vector3d::UnitZ());
    RotAxisB = Eigen::AngleAxisd(M_PI_2 - D[1], vB[1]);
    A[1] = P + Rot * B[2] + cableLength[2] * Rot * RotAxisB * RotZ * vB[1];

    // compute A43
    RotZ = Eigen::AngleAxisd(beta[2], Eigen::Vector3d::UnitZ());
    RotAxisB = Eigen::AngleAxisd(M_PI_2 - D[2], vB[2]);
    A[2] = P + Rot * B[4] + cableLength[4] * Rot * RotAxisB * RotZ * vB[2];

    conf->setQuadAnchors(A);
}


/**
 * Cost function involving the tensions exerted on the cables and the quadrotor thrusts.
 */
double CostSpace::flyCraneCost(Configuration * conf)
{
    // coordinates of the anchor points of the quadrotors, in the global reference frame
    if (!conf->getAssignedQuadAnchors())
        this->computeQuadAnchorPoints(conf);
    Eigen::Vector3d * A = conf->getQuadAnchors();

    // position and orientation (as a rotation matrix) of the platform
    Eigen::Vector3d P = conf->getPosition();
    Eigen::Matrix3d Rot = this->RotationMatrix(conf->getOrientation());

    // unit vectors of the cables: Ui = (gBi - Ai) / ropeLength[i]
    // where gBi = P + Rot * Bi are the coordinates of the platform anchor points in the global reference frame
    Eigen::Vector3d U0 = ((P + Rot * B[0]) - A[0]).normalized();
    Eigen::Vector3d U1 = ((P + Rot * B[1]) - A[1]).normalized();
    Eigen::Vector3d U2 = ((P + Rot * B[2]) - A[1]).normalized();
    Eigen::Vector3d U3 = ((P + Rot * B[3]) - A[2]).normalized();
    Eigen::Vector3d U4 = ((P + Rot * B[4]) - A[2]).normalized();
    Eigen::Vector3d U5 = ((P + Rot * B[5]) - A[0]).normalized();

	// moments of the Ui vectors wrt the origin of the global reference frame ri = Ui x (-Ai) = Ai x Ui
	Eigen::Vector3d r0 = A[0].cross(U0);
	Eigen::Vector3d r1 = A[1].cross(U1);
	Eigen::Vector3d r2 = A[1].cross(U2);
	Eigen::Vector3d r3 = A[2].cross(U3);
	Eigen::Vector3d r4 = A[2].cross(U4);
	Eigen::Vector3d r5 = A[0].cross(U5);

    // screw Jacobian of the manipulator (6x6), composed of the Ui vectors and
    // their moments with respect to the origin of the global reference frame: ri = Ui x (-Ai) = Ai x Ui
    Matrix6d J;
    J << U0, U1, U2, U3, U4, U5, r0, r1, r2, r3, r4, r5;

    // if the determinant of the Jacobian is zero, return infinite cost
    if (abs(J.determinant()) > EPSILON)
    {
        // coordinates of the point where the wrench is exerted (expressed in the global reference frame)
        Eigen::Vector3d gWP = P + Rot * wrenchPoint;

        // CW: change of reference frame for the wrench
        Eigen::Matrix3d aux;
        aux << 0, -gWP(2), gWP(1), gWP(2), 0, -gWP(0), -gWP(1), gWP(0), 0;
        Matrix6d CW;
        CW << Eigen::Matrix3d::Identity(), Eigen::Matrix3d::Zero(), aux, Eigen::Matrix3d::Identity();

        // cable tension vector corresponding to the given wrench
        Vector6d T0 = J.inverse() * CW * wrench;

        // JtEJ: ellipsoid in the tension space
        Matrix6d CW1 = CW.inverse();
        Matrix6d JtEJ = J.transpose() * CW1.transpose() * Ellipsoid * CW1 * J;

        // for each cable, compute the "distance" between the current tension, expressed as an interval
        // because of uncertainties, and the limits of the admissible tension interval
        double maximalTensions[6], minimalTensions[6];
        double cost(0.);
        for (unsigned r(0); r < 6; ++r)
        {
            // sub-matrix of JtEJ without the r-th row
            Matrix56d EEr;
            for (unsigned j(0); j < 6; ++j)
                for (unsigned i(0); i < 5; ++i)
                    EEr(i, j) = (i < r) ? JtEJ(i, j) : JtEJ(i + 1, j);

            // the columns of V corresponding to 0 singular values (i.e. to 0 in the diagonal of svd)
            // are a basis of ker(EEr)
            Eigen::JacobiSVD<Eigen::MatrixXd> svd(EEr, Eigen::ComputeFullV);

            // therefore we are interested in the last column of V
            Vector6d V5 = svd.matrixV().col(5);

            // offset of the tension region in the r-th cable
            double value = V5.transpose() * JtEJ * V5;
            double offset_r = abs(V5(r) / sqrt(value));

            // distances of the minimum and maximum tension values of the r-th cable in this ellipsoid
            // to the bounds of the admissible tension interval
            maximalTensions[r] = T0(r) + offset_r;
            minimalTensions[r] = T0(r) - offset_r;
            double dist_min = minimalTensions[r] - admissibleTension[r][0];
            double dist_max = admissibleTension[r][1] - maximalTensions[r];

            // if the tension is not admissible, the cost is infinite
            if (dist_min <= 0. || dist_max <= 0.)
            	cost = std::numeric_limits<double>::max();

            // otherwise, update the cost by adding to it
            // the inverse of the distances normalized by the size of the admissible tension interval
            else {
                double tensionInterval = admissibleTension[r][1] - admissibleTension[r][0];
                cost += tensionInterval / dist_min + tensionInterval / dist_max;
            }
        }

		// resultant cable forces (in the global reference frame)
        Eigen::Vector3d cForces[3];
		cForces[0] = T0(0) * U0 + T0(5) * U5;
		cForces[1] = T0(1) * U1 + T0(2) * U2;
		cForces[2] = T0(3) * U3 + T0(4) * U4;

		// resultant forces applied on each quadrotor (in the global reference frame) and their squared norm
		Eigen::Vector3d qF[3];
		double sqrNormQF[3];
		for (unsigned i(0); i < 3; ++i) {
			qF[i] = cForces[i] + quadWeight[i];
			sqrNormQF[i] = qF[i].squaredNorm();
		}

		// offset vectors CMij - Aij (in the global reference frame; recall that qForces point "downwards")
		Eigen::Vector3d offsetV[3];
		for (unsigned i(0); i < 3; ++i)
			offsetV[i] = - offsetAnchorCM[i] * qF[i].normalized();

		// coordinates of the center of mass of each quadrotor (in the global reference frame)
		Eigen::Vector3d CM[3];
		for (unsigned i(0); i < 3; ++i)
			CM[i] = A[i] + offsetV[i];
        conf->setCentersOfMass(CM);

		// couples associated to the qForces corresponding to their wrenches (in the global reference frame)
		Eigen::Vector3d couple[3];
		couple[0] = T0(0) * r0 + T0(5) * r5 + cForces[0].cross(offsetV[0]) + CM[0].cross(quadWeight[0]);
		couple[1] = T0(1) * r1 + T0(2) * r2 + cForces[1].cross(offsetV[1]) + CM[1].cross(quadWeight[1]);
		couple[2] = T0(3) * r3 + T0(4) * r4 + cForces[2].cross(offsetV[2]) + CM[2].cross(quadWeight[2]);

		// vector distances of the applied qForces to the CMs
		Eigen::Vector3d qF_CM[3];
		for (unsigned i(0); i < 3; ++i)
			qF_CM[i] = qF[i].cross(couple[i]) + qF[i].dot(CM[i]) * qF[i] - sqrNormQF[i] * CM[i];

		// offsets of the application points of the forces wrt the maximum allowed distance
		double dfCM05 = dCMmotor[0] / M_SQRT2 - qF_CM[0].norm() / sqrNormQF[0];
		double dfCM12 = dCMmotor[1] / M_SQRT2 - qF_CM[1].norm() / sqrNormQF[1];
		double dfCM34 = dCMmotor[2] / M_SQRT2 - qF_CM[2].norm() / sqrNormQF[2];

		if (dfCM05 <= 0. || dfCM12 <= 0. || dfCM34 <= 0.)
			// if the application point of a force is outside the valid zone, the cost is infinite
			cost = std::numeric_limits<double>::max();
		else
            // otherwise, update the cost by adding to it the normalized inverse of the distances
			cost += dCMmotor[0] / (M_SQRT2 * dfCM05) + dCMmotor[1] / (M_SQRT2 * dfCM12)
					+ dCMmotor[2] / (M_SQRT2 * dfCM34);

        // compute the forces exerted by each quadrotor
        Eigen::Vector3d forces05mxmx = maximalTensions[0] * U0 + maximalTensions[5] * U5 + quadWeight[0];
        Eigen::Vector3d forces05mnmx = minimalTensions[0] * U0 + maximalTensions[5] * U5 + quadWeight[0];
        Eigen::Vector3d forces05mxmn = maximalTensions[0] * U0 + minimalTensions[5] * U5 + quadWeight[0];
        double force05max = max(forces05mxmx.norm(), max(forces05mnmx.norm(), forces05mxmn.norm()));

        Eigen::Vector3d forces12mxmx = maximalTensions[1] * U1 + maximalTensions[2] * U2 + quadWeight[1];
        Eigen::Vector3d forces12mnmx = minimalTensions[1] * U1 + maximalTensions[2] * U2 + quadWeight[1];
        Eigen::Vector3d forces12mxmn = maximalTensions[1] * U1 + minimalTensions[2] * U2 + quadWeight[1];
        double force12max = max(forces12mxmx.norm(), max(forces12mnmx.norm(), forces12mxmn.norm()));

        Eigen::Vector3d forces34mnmx = minimalTensions[3] * U3 + maximalTensions[4] * U4 + quadWeight[2];
        Eigen::Vector3d forces34mxmx = maximalTensions[3] * U3 + maximalTensions[4] * U4 + quadWeight[2];
        Eigen::Vector3d forces34mxmn = maximalTensions[3] * U3 + minimalTensions[4] * U4 + quadWeight[2];
        double force34max = max(forces34mxmx.norm(), max(forces34mnmx.norm(), forces34mxmn.norm()));

        // update the configuration with the values of tensions and forces
        double quadForces[3] = { force05max, force12max, force34max };
        conf->setTensionsAndForces(maximalTensions, minimalTensions, quadForces);

        // if forces are not acceptable, return infinite cost
        double df05 = quadThrust[0] - force05max;
        double df21 = quadThrust[1] - force12max;
        double df43 = quadThrust[2] - force34max;

        if (df05 <= 0. || df21 <= 0. || df43 <= 0.)
            return std::numeric_limits<double>::max();

        // if forces are acceptable, update the cost by adding to it the normalized inverse of the offsets
        cost += quadThrust[0] / df05 + quadThrust[1] / df21 + quadThrust[2] / df43;
        return cost;
    }
    else
        return std::numeric_limits<double>::max();
}


/**
 * FlyCrane cost function.
 */
double computeFlyCranecost(Configuration& conf)
{
    if (conf.isInCollision())
        return numeric_limits<double>::max();

    conf.initializeFlyCraneConfiguration();
    return (global_costSpace->flyCraneCost(&conf));
}


//----------------------------------------------------------------------
// Basic cost functions
//----------------------------------------------------------------------

double computeFlatCost(Configuration& conf)
{
	return 1.0;
}

extern void* GroundCostObj;

double computeIntersectionWithGround(Configuration& conf)
{
	double cost(0);
	if(GroundCostObj)
	{
		GHintersectionVerticalLineWithGround(GroundCostObj, 
																				 conf.getConfigStruct()[6],
																				 conf.getConfigStruct()[7], 
																				 &cost);
	}
  //cout << "Ground Cost = " << cost << endl;
	return(cost);
}

double computeClearance(Configuration & conf)
{
    if (conf.isInCollision())
        return numeric_limits<double>::max();

    Robot * robot = conf.getRobot();
    string robotName(robot->getRobotStruct()->name);
    robot->setAndUpdate(conf);

    if (robotName.compare("gt6ag") == 0)
    	return 1000. / p3d_GetMinDistCost(robot->getRobotStruct());

    else
        return 100. / p3d_GetMinDistCost(robot->getRobotStruct());
}


double computeDistanceToObstacles(Configuration& conf)
{
    if (conf.isInCollision())
        return numeric_limits<double>::max();

    Robot* robot = conf.getRobot();
    robot->setAndUpdate(conf);
    return p3d_GetMinDistCost(robot->getRobotStruct());
}


double computeInCollisionCost(Configuration& conf)
{
	Robot* robot = conf.getRobot();
	shared_ptr<Configuration> qActual = robot->getCurrentPos();
  
  double cost = 0.1;
  
  if( conf.isInCollision() )
  {
    cost = 20.0;
  }
  
	robot->setAndUpdate(*qActual);
	return cost;
}

double computeCollisionSpaceCost(Configuration& conf)
{
  double cost = 0.1;
  if( optimizer.get() != NULL ) {
    cost = PlanEnv->getDouble(PlanParam::trajOptimObstacWeight)*optimizer->getCollisionSpaceCost( conf );
  }
	return cost;
}

double computeLocalpathKinematicCost(p3d_rob* rob, p3d_localpath* LP)
{
  if (LP == NULL) {
    return 1;
  }
  
  Robot* currRob = global_Project->getActiveScene()->getRobotByNameContaining(rob->name);
  LocalPath path(currRob,LP);
  double cost = path.cost();
  cout << "Kinematic cost = " << cost << endl;
  return cost;
}


std::vector<double> robotToRobotvector(p3d_rob* robot1,p3d_rob* robot2)
{

    std::vector<double> robTorob;
    p3d_col_activate_rob_rob( robot1, robot2 );

    int nof_bodies = robot1->no;
    double* distances = new double[nof_bodies];
    p3d_vector3 *body = new p3d_vector3[nof_bodies];
    p3d_vector3 *other = new p3d_vector3[nof_bodies];

    int k=0;
    double minDist = std::numeric_limits<double>::max();

    // Checkout the collision checker version
    switch (p3d_col_get_mode())
    {
        case p3d_col_mode_kcd:
        {
            set_kcd_which_test((p3d_type_col_choice)P3D_KCD_ROB_ROB);
            p3d_col_test_choice();

            p3d_kcd_closest_points_between_bodies(robot1,body,other,distances);
            for(int i=0;i<nof_bodies;i++)
            {
                if( minDist > distances[i] )
                {
                    minDist = distances[i];
                    k = i;
                }
            }

            break;
        }
        case p3d_col_mode_pqp:
        {
            minDist =  pqp_robot_robot_distance(robot1,robot2, body[k], other[k]);
            break;
        }
    }

    robTorob.clear();
    robTorob.push_back(body[k][0] - other[k][0]);
    robTorob.push_back(body[k][1] - other[k][1]);
    robTorob.push_back(body[k][2] - other[k][2]);


    delete[] distances;
    delete[] other;
    delete[] body;

    return robTorob;

}

double computeHotCost(Configuration& conf)
{
    //First we make sure that the robot is carrying something, if not, we use a default cost.
    if(!conf.getRobot()->getRobotStruct()->isCarryingObject)
    {
        return computeDistanceToObstacles(conf);
    }
    //If the robot is carrying something, we compute the different part of the cost

    double cost = numeric_limits<double>::max();


    double angleToHuman = 180; //the worst situation is assumed
    double distanceToHuman = 0;


    double paramDistanceToHuman = ENV.getDouble(Env::paramDistanceToHuman);
    double paramAngleToHuman = ENV.getDouble(Env::paramAngleToHuman);


    //Hot face related information computation
    std::vector<ArmManipulationData>* armData = conf.getRobot()->getRobotStruct()->armManipulationData;
    p3d_rob* carriedObject = armData->at(0).getCarriedObject();

    poly_plane* hotPlane = carriedObject->ocur->polcur->poly->the_faces[0].plane ;//The first plane defined in the p3d file is the dangerous one (if there is more than one dangerous plane ???)

    //In order to get the normal vector to the plane corresponding to the dangerous face we have to move the orginal normale to the current position of the object
    double x,y,z,cx,sx,cy,sy,cz,sz;

    x = hotPlane->normale[0];
    y = hotPlane->normale[1];
    z = hotPlane->normale[2];

    cx = cos(conf.getRobot()->getJoint("virtual_object_right")->getJointDof(3));
    sx = sin(conf.getRobot()->getJoint("virtual_object_right")->getJointDof(3));

    cy = cos(conf.getRobot()->getJoint("virtual_object_right")->getJointDof(4));
    sy = sin(conf.getRobot()->getJoint("virtual_object_right")->getJointDof(4));

    cz = cos(conf.getRobot()->getJoint("virtual_object_right")->getJointDof(5));
    sz = sin(conf.getRobot()->getJoint("virtual_object_right")->getJointDof(5));

    poly_vector3 hotNormale;

    hotNormale[0] =  x*cy*cz + y*( sx*sy*cz + cx*sz) + z*(-cx*sy*cz + sx*sz);
    hotNormale[1] = -x*cy*sz + y*(-sx*sy*sz + cx*cz) + z*( cx*sy*sz + sx*cz);
    hotNormale[2] =  x*sy    + y*(-sx*cy)            + z*(cx*cy);

    double hotNorm = sqrt(pow(hotNormale[0],2.0)+pow(hotNormale[1],2.0)+pow(hotNormale[2],2.0));

    //Human related data


    p3d_rob* human = global_Project->getActiveScene()->getRobotByNameContaining("HERAKLES")->getRobotStruct();
    std::vector<double> humanToObject = robotToRobotvector(human,carriedObject);

    double humNorm = sqrt(pow(humanToObject[0],2.0)+pow(humanToObject[1],2.0)+pow(humanToObject[2],2.0));

    distanceToHuman = humNorm;

    //We compute angleToHuman
    double prodsca = hotNormale[0]*humanToObject[0] + hotNormale[1]*humanToObject[1] + hotNormale[2]*humanToObject[2];

    angleToHuman = abs((acos(prodsca/(hotNorm*humNorm)) * 180.0 / 3.14159265));

    if(angleToHuman > 120 && angleToHuman < 135)
    {    angleToHuman *= 5;}
    else if(angleToHuman > 135)
    {     angleToHuman *= 10; }



    cost = exp(((paramAngleToHuman*angleToHuman/180)) * (1 + paramDistanceToHuman/distanceToHuman));

    return cost;
}


//----------------------------------------------------------------------
void CostSpace::initMotionPlanning(p3d_graph* graph, p3d_node* start, p3d_node* goal)
{
  Robot* robot = global_Project->getActiveScene()->getRobotByName( graph->rob->name );
	start->temp = ENV.getDouble(Env::initialTemperature);
	start->comp->temperature = ENV.getDouble(Env::initialTemperature);
	start->nbFailedTemp = 0;
	
	p3d_SetGlobalNumberOfFail(0);
	
  Configuration qStart( robot, start->q );
	p3d_SetNodeCost( graph, start, qStart.cost() );
	p3d_SetCostThreshold( start->cost );
	
#ifdef P3D_PLANNER
	p3d_SetInitCostThreshold( start->cost );
#else
	printf("P3D_PLANNER not compiled in %s in %s",__func__,__FILE__);
#endif
	
	if ( ENV.getBool(Env::expandToGoal) && (goal != NULL))
	{
		goal->temp					= ENV.getDouble(Env::initialTemperature);
		goal->comp->temperature	= ENV.getDouble(Env::initialTemperature);
		start->temp				= ENV.getDouble(Env::initialTemperature);
		goal->nbFailedTemp = 0;
		//    Ng->NbDown = 0;
    
    Configuration qGoal( robot, goal->q );
		p3d_SetNodeCost( graph, goal, qGoal.cost());
		p3d_SetCostThreshold(MAX(qStart.cost(), qGoal.cost() ));
		
		//        p3d_SetCostThreshold(MAX(
		//								p3d_GetNodeCost(this->getStart()->getNodeStruct()), 
		//								p3d_GetNodeCost(this->getGoal()->getNodeStruct()) ));
		
		p3d_SetAverQsQgCost(
												( graph->search_start->cost
												 +graph->search_goal->cost) / 2.);
	}
	else
	{
#ifdef P3D_PLANNER
		p3d_SetCostThreshold( start->cost );
		p3d_SetInitCostThreshold(start->cost );
		p3d_SetAverQsQgCost( graph->search_start->cost );
#else
		printf("P3D_PLANNER not compiled in %s in %s",__func__,__FILE__);
#endif
	}
}
